@@ -1,3 +1,5 @@
+from zope.app.publication.browser import setDefaultSkin
+from zope.publisher.interfaces.browser import IDefaultBrowserLayer
 from ZPublisher import Retry
 from ZODB.POSException import ConflictError
 
@@ -118,6 +120,14 @@ class Request:
         r.retry_count = self.retry_count
         return r
 
+class RequestWithSkinCheck(Request):
+    def traverse(self, path, validated_hook):
+        if IDefaultBrowserLayer.providedBy(self):
+            return Object()
+        else:
+            tracer.exceptions['__call__'] = [ValueError]
+            return Object()
+
 module_name = __name__
 after_list = [None]
 
@@ -263,6 +273,115 @@ def testPublisher():
     raising ConflictError from zpublisher_exception_hook
     abort
 
+    The request generator applies the default skin layer to the request.
+    We have a specially crafted request that tests this.  If the
+    request does not have the required interface it raises an
+    ValueError.  Let's see that this works as expected
+
+    >>> tracer.reset()
+    >>> request = RequestWithSkinCheck()
+    >>> setDefaultSkin(request)
+    >>> response = publish(request, module_name, after_list)
+    >>> tracer.showTracedPath()
+    begin
+    __call__
+    commit
+
+    Retries generate new request objects, the publisher needs to
+    ensure that the skin layer is applied to those as well. If the
+    skin layer is not applied to subsequent requests, an ValueError
+    would be raised here.
+
+    >>> tracer.reset()
+    >>> tracer.exceptions['commit'] = [ConflictError, ConflictError,
+    ...                                  ConflictError, ConflictError]
+    >>> request = RequestWithSkinCheck()
+    >>> setDefaultSkin(request)
+    >>> response = publish(request, module_name, after_list)
+    Traceback (most recent call last):
+    ...
+    ConflictError: database conflict error
+    >>> tracer.showTracedPath()
+    begin
+    __call__
+    commit
+    raising ConflictError from commit
+    abort
+    begin
+    __call__
+    commit
+    raising ConflictError from commit
+    abort
+    begin
+    __call__
+    commit
+    raising ConflictError from commit
+    abort
+    begin
+    __call__
+    commit
+    raising ConflictError from commit
+    abort
+
+    """
+    pass
+
+class ObjectNotFound:
+    """Mock object for traversing to.
+    """
+
+    def __call__(self):
+        tracer.append('ObjectNotFound')
+        return 'ObjectNotFound'
+
+
+class PathRequest(Request):
+    def __init__(self, path):
+        self.PATH_INFO = path
+        Request.__init__(self)
+
+    def get(self, a, b=''):
+        if a == 'PATH_INFO':
+            return self.PATH_INFO
+        else:
+            return ''
+
+    def traverse(self, path, validated_hook):
+        if path == self.PATH_INFO:
+            return Object()
+        else:
+            return ObjectNotFound()
+
+def testPublishPath():
+    """
+    Tests to ensure that publish passes paths through to the request without
+    stripping spaces (as this can lead to google indexing pages with a trailing
+    space when someone has a typo in an href to you're site). Zope bug #1991.
+
+    >>> from ZPublisher.Publish import publish
+
+    Without the trailing space, should work normally
+
+    >>> tracer.reset()
+    >>> request = PathRequest('/foo')
+    >>> response = publish(request, module_name, after_list)
+    >>> tracer.showTracedPath()
+    begin
+    __call__
+    commit
+
+    Now with a trailing space, should also work normally, but in zope 2.9.0
+    and earlier publish did a strip() on the path so instead of __call__ you
+    an ObjectNotFound in the trace.
+
+    >>> tracer.reset()
+    >>> request = PathRequest('/foo ')
+    >>> response = publish(request, module_name, after_list)
+    >>> tracer.showTracedPath()
+    begin
+    __call__
+    commit
+
     """
     pass
 
