@@ -17,9 +17,10 @@ Note, for a detailed description of the way that conflicting
 configuration actions are resolved, see the detailed example in
 test_includeOverrides in tests/text_xmlconfig.py
 
-$Id: xmlconfig.py 27081 2004-08-12 19:56:31Z srichter $
+$Id: xmlconfig.py 68962 2006-07-04 01:26:28Z tseaver $
 """
 __docformat__ = 'restructuredtext'
+
 import errno
 import os
 import sys
@@ -38,6 +39,10 @@ from zope.interface import Interface
 
 logger = logging.getLogger("config")
 
+ZCML_NAMESPACE = "http://namespaces.zope.org/zcml"
+ZCML_CONDITION = (ZCML_NAMESPACE, u"condition")
+
+
 class ZopeXMLConfigurationError(ConfigurationError):
     """Zope XML Configuration error
 
@@ -140,7 +145,7 @@ class ParserInfo(object):
         file = self.file
         if file == 'tests//sample.zcml':
             # special case for testing
-            file = os.path.join(os.path.split(__file__)[0],
+            file = os.path.join(os.path.dirname(__file__),
                                 'tests', 'sample.zcml')
 
         try:
@@ -163,7 +168,7 @@ class ParserInfo(object):
             if lines[0][:column].strip():
                 # Remove text before start if it's noy whitespace
                 lines[0] = lines[0][self.column:]
-                
+
             src = ''.join([u"  "+l for l in lines])
 
         return "%s\n%s" % (`self`, src)
@@ -181,6 +186,7 @@ class ConfigurationHandler(ContentHandler):
     def __init__(self, context, testing=0):
         self.context = context
         self.testing = testing
+        self.ignore_depth = 0
 
     def setDocumentLocator(self, locator):
         self.locator = locator
@@ -189,12 +195,22 @@ class ConfigurationHandler(ContentHandler):
         self.context.getInfo().characters(text)
 
     def startElementNS(self, name, qname, attrs):
+        if self.ignore_depth:
+            self.ignore_depth += 1
+            return
 
         data = {}
         for (ns, aname), value in attrs.items():
             if ns is None:
                 aname = str(aname)
                 data[aname] = value
+            if (ns, aname) == ZCML_CONDITION:
+                # need to process the expression to determine if we
+                # use this element and it's descendents
+                use = self.evaluateCondition(value)
+                if not use:
+                    self.ignore_depth = 1
+                    return
 
         info = ParserInfo(
             self.locator.getSystemId(),
@@ -204,17 +220,104 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.begin(name, data, info)
+        except (KeyboardInterrupt, SystemExit):
+            raise
         except:
             if self.testing:
                 raise
-            raise ZopeXMLConfigurationError, (
-                info, sys.exc_info()[0], sys.exc_info()[1]
-                ), sys.exc_info()[2]
+            raise ZopeXMLConfigurationError(info, sys.exc_info()[0],
+                sys.exc_info()[1]), None, sys.exc_info()[2]
 
         self.context.setInfo(info)
 
+    def evaluateCondition(self, expression):
+        """Evaluate a ZCML condition.
+
+        `expression` is a string of the form "verb arguments".
+
+        Currently the two supported verba are 'have' and 'installed'.
+
+        The 'have' verb takes one argument: the name of a feature.
+
+        >>> from zope.configuration.config import ConfigurationContext
+        >>> context = ConfigurationContext()
+        >>> context.provideFeature('apidoc')
+        >>> c = ConfigurationHandler(context, testing=True)
+        >>> c.evaluateCondition("have apidoc")
+        True
+        >>> c.evaluateCondition("have onlinehelp")
+        False
+
+        Ill-formed expressions raise an error
+
+        >>> c.evaluateCondition("want apidoc")
+        Traceback (most recent call last):
+          ...
+        ValueError: Invalid ZCML condition: 'want apidoc'
+
+        >>> c.evaluateCondition("have x y")
+        Traceback (most recent call last):
+          ...
+        ValueError: Only one feature allowed: 'have x y'
+
+        >>> c.evaluateCondition("have")
+        Traceback (most recent call last):
+          ...
+        ValueError: Feature name missing: 'have'
+
+
+        The 'installed' verb takes one argument: the dotted name of a
+        pacakge. If the pacakge is found, in other words, can be imported,
+        then the condition will return true.
+
+        >>> from zope.configuration.config import ConfigurationContext
+        >>> context = ConfigurationContext()
+        >>> c = ConfigurationHandler(context, testing=True)
+        >>> c.evaluateCondition('installed zope.interface')
+        True
+        >>> c.evaluateCondition('installed zope.foo')
+        False
+
+        Ill-formed expressions raise an error
+
+        >>> c.evaluateCondition("installed foo bar")
+        Traceback (most recent call last):
+          ...
+        ValueError: Only one package allowed: 'installed foo bar'
+
+        >>> c.evaluateCondition("installed")
+        Traceback (most recent call last):
+          ...
+        ValueError: Package name missing: 'installed'
+        """
+        arguments = expression.split(None)
+        verb = arguments.pop(0)
+        if verb == 'have':
+            if not arguments:
+                raise ValueError("Feature name missing: %r" % expression)
+            if len(arguments) > 1:
+                raise ValueError("Only one feature allowed: %r" % expression)
+            return self.context.hasFeature(arguments[0])
+        elif verb == 'installed':
+            if not arguments:
+                raise ValueError("Package name missing: %r" % expression)
+            if len(arguments) > 1:
+                raise ValueError("Only one package allowed: %r" % expression)
+            try:
+                __import__(arguments[0])
+            except ImportError:
+                return False
+            return True
+        else:
+            raise ValueError("Invalid ZCML condition: %r" % expression)
 
     def endElementNS(self, name, qname):
+        # If ignore_depth is set, this element will be ignored, even
+        # if this this decrements ignore_depth to 0.
+        if self.ignore_depth:
+            self.ignore_depth -= 1
+            return
+
         info = self.context.getInfo()
         info.end(
             self.locator.getLineNumber(),
@@ -223,12 +326,13 @@ class ConfigurationHandler(ContentHandler):
 
         try:
             self.context.end()
+        except (KeyboardInterrupt, SystemExit):
+            raise
         except:
             if self.testing:
                 raise
-            raise ZopeXMLConfigurationError, (
-                info, sys.exc_info()[0], sys.exc_info()[1]
-                ), sys.exc_info()[2]
+            raise ZopeXMLConfigurationError(info, sys.exc_info()[0],
+                sys.exc_info()[1]), None, sys.exc_info()[2]
 
 
 def processxmlfile(file, context, testing=False):
@@ -244,7 +348,7 @@ def processxmlfile(file, context, testing=False):
     try:
         parser.parse(src)
     except SAXParseException:
-        raise ZopeSAXParseException, sys.exc_info()[1], sys.exc_info()[2]
+        raise ZopeSAXParseException(sys.exc_info()[1]), None, sys.exc_info()[2]
 
 
 def openInOrPlain(filename):
@@ -262,7 +366,7 @@ def openInOrPlain(filename):
 
     If we open configure.zcml, we'll get that file:
 
-    >>> here = os.path.split(__file__)[0]
+    >>> here = os.path.dirname(__file__)
     >>> path = os.path.join(here, 'tests', 'samplepackage', 'configure.zcml')
     >>> f = openInOrPlain(path)
     >>> f.name[-14:]
@@ -276,6 +380,19 @@ def openInOrPlain(filename):
     >>> f.name[-11:]
     'foo.zcml.in'
 
+    Make sure other IOErrors are re-raised.  We need to do this in a
+    try-except block because different errors are raised on Windows and
+    on Linux.
+
+    >>> try:
+    ...     f = openInOrPlain('.')
+    ... except IOError:
+    ...     print "passed"
+    ... else:
+    ...     print "failed"
+    ...
+    passed
+
     """
     try:
         fp = open(filename)
@@ -286,6 +403,8 @@ def openInOrPlain(filename):
                 fp = open(fn)
             else:
                 raise
+        else:
+            raise
     return fp
 
 class IInclude(Interface):
@@ -306,19 +425,19 @@ class IInclude(Interface):
 
     files = schema.BytesLine(
         title=u"Configuration file name pattern",
-        description=u"""\
+        description=u"""
         The names of multiple configuration files to be included,
         expressed as a file-name pattern, relative to the directive
         containing the including configuration file.  The pattern can
         include:
 
         - ``*`` matches 0 or more characters
-        
+
         - ``?`` matches a single character
-        
-        - ``[<seq>]`` matches any character in seq 
-        
-        - ``[!<seq>]`` matches any character not in seq 
+
+        - ``[<seq>]`` matches any character in seq
+
+        - ``[!<seq>]`` matches any character not in seq
 
         The file names are included in sorted order, where sorting is
         without regard to case.
@@ -328,9 +447,9 @@ class IInclude(Interface):
 
     package = config.fields.GlobalObject(
         title=u"Include package",
-        description=u"""\
-        Include the named file (or configure.zcml)
-        from the directory of this package.
+        description=u"""
+        Include the named file (or configure.zcml) from the directory of this
+        package.
         """,
         required=False,
         )
@@ -362,13 +481,13 @@ def include(_context, file=None, package=None, files=None):
         paths = [path for (l, path) in paths]
     else:
         paths = [context.path(file)]
-    
+
     for path in paths:
         if context.processFile(path):
             f = openInOrPlain(path)
             logger.debug("include %s" % f.name)
 
-            context.basepath = os.path.split(path)[0]
+            context.basepath = os.path.dirname(path)
             context.includepath = _context.includepath + (f.name, )
             _context.stack.append(config.GroupingStackItem(context))
 
@@ -377,7 +496,7 @@ def include(_context, file=None, package=None, files=None):
             assert _context.stack[-1].context is context
             _context.stack.pop()
 
-def includeOverrides(_context, file, package=None):
+def includeOverrides(_context, file=None, package=None, files=None):
     """Include zcml file containing overrides
 
     The actions in the included file are added to the context as if they
@@ -394,7 +513,7 @@ def includeOverrides(_context, file, package=None):
     includepath = _context.includepath
 
     # Now we'll include the file. We'll munge the actions after
-    include(_context, file, package)
+    include(_context, file, package, files)
 
     # Now we'll grab the new actions, resolve conflicts,
     # and munge the includepath:
