@@ -13,11 +13,11 @@
 ******************************************************************************
 Security Proxy Implementation
 
-$Id: _proxy.c 26705 2004-07-23 16:22:56Z jim $
+$Id: _proxy.c 37889 2005-08-12 15:25:08Z fdrake $
 */
 
 #include <Python.h>
-#include "proxy.h"
+#include "zope.proxy/proxy.h"
 
 static PyObject *__class__str = 0, *__name__str = 0, *__module__str = 0;
 
